/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== checkbox.css ===================================================
  == Styles used by the XUL checkbox element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: checkbox ::::: */

checkbox {
  -moz-appearance: checkbox-container;
  margin: 2px 4px;
  border-left: 2px transparent;
  border-right: 2px transparent;
}

/* With native theming on, the spacer-box paints the check mark and includes
   the spacing to the right, so that oversized checkbox images can slop over
   into the space.

   If we need to disable checkbox theming, the spacer box no longer paints
   the check mark, but its fallback CSS border supplies the padding between
   the mark and the label.  The xul:image then takes over painting the check
   mark. */

.checkbox-spacer-box {
  -moz-appearance: checkbox;
  -moz-box-align: center;
  border-right: 4px solid transparent;
}

.checkbox-label-center-box {
  -moz-box-align: center;
}

.checkbox-label-box {
  border: 1px solid transparent;
  -moz-appearance: checkbox-label;
}

.checkbox-icon[src] {
  -moz-margin-end: 2px;
}

.checkbox-label {
  margin: 0 !important;
}

/* ..... focused state ..... */

checkbox:focus > .checkbox-label-center-box > .checkbox-label-box {
  border: 1px dotted ThreeDDarkShadow;
}

/* ..... disabled state ..... */

checkbox[disabled="true"] > .checkbox-spacer-box > .checkbox-check {
  background-color: -moz-Dialog;
}

checkbox[disabled="true"] {
  color: GrayText !important;
}

/* ::::: checkmark image ::::: */

.checkbox-check {
  border: 2px solid;
  -moz-border-top-colors: ThreeDShadow ThreeDDarkShadow;
  -moz-border-right-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-bottom-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-left-colors: ThreeDShadow ThreeDDarkShadow;
  min-width: 13px;
  min-height: 13px;
  background: -moz-Field no-repeat 50% 50%;
}

checkbox:hover {
  color: -moz-buttonhovertext;
}

checkbox:hover:active > .checkbox-spacer-box > .checkbox-check {
  background-color: -moz-Dialog;
}

/* ..... checked state ..... */

checkbox[checked="true"] > .checkbox-spacer-box > .checkbox-check {
  background-image: url("chrome://global/skin/checkbox/cbox-check.gif");
}

checkbox[checked="true"][disabled="true"] > .checkbox-spacer-box > .checkbox-check {
  background-image: url("chrome://global/skin/checkbox/cbox-check-dis.gif") !important
}

