/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt (hewitt@netscape.com)
 *   Kevin Gerich (webmail@kmgerich.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== tree.css ===================================================
  == Styles used by the XUL outline element.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

/* ::::: tree ::::: */

tree {
  margin: 0px 4px;
  border-top: 1px solid #A1A1A1;
  border-right: 1px solid #C3C3C3;
  border-bottom: 1px solid #DDDDDD;
  border-left: 1px solid #C3C3C3;
  color: -moz-DialogText;
  background-color: #FFFFFF;
}

/* ::::: tree focusring ::::: */

.focusring > .tree-rows > .tree-bodybox {
  border: 1px solid transparent;  
}

.focusring:focus > .tree-rows > .tree-bodybox {
  border: 1px solid -moz-mac-focusring;
}


/* ::::: tree rows ::::: */

treechildren::-moz-tree-row {
  border-top: 1px solid -moz-field;
  height: 18px;
  background-color: #fff;
}

tree:not([hidecolumnpicker="true"]) treechildren::-moz-tree-row(odd) {
 background-color: #f0f0f0;
}

treechildren::-moz-tree-row(selected),
treechildren::-moz-tree-row(odd, selected),
tree:not([hidecolumnpicker="true"]) treechildren::-moz-tree-row(odd, selected) {
  background-color: #D0D0D0;
}

treechildren::-moz-tree-row(selected, focus), 
treechildren::-moz-tree-row(odd, selected, focus),
tree:not([hidecolumnpicker="true"]) treechildren::-moz-tree-row(odd, selected, focus) {
  /* background-color: Highlight; */
  background-color: #3874D1;
  color: #FFFFFF;
}

/* ::::: tree cells ::::: */

treechildren::-moz-tree-cell {
  padding: 0px 2px 0px 2px;
}

treechildren::-moz-tree-cell-text {
  color: inherit;
}

treechildren::-moz-tree-cell-text(selected) {
  color: -moz-DialogText;
}

treechildren::-moz-tree-cell-text(selected, focus) {
  /* color: HighlightText; */
  color: #FFFFFF;
}


/* ::::: lines connecting cells ::::: */

treechildren::-moz-tree-line {
  /* XXX there should be no border on Mac, but trees currently
         paint the line black by default, so I'll just leave this
         for now. */
  visibility: hidden;
  border: 1px dotted grey;
}


/* ::::: tree separator ::::: */

treechildren::-moz-tree-separator {
  border-top: 1px dashed #C7C7C7;
  margin: 0 2px;
}


/* ::::: drop feedback ::::: */

treechildren::-moz-tree-cell(primary, dropOn) {
  background-color: #A1A1A1 !important;
  color: #FFF !important;
  background-image: none;
}
treechildren::-moz-tree-cell-text(primary, dropOn) {
  color: #FFF !important;
}

treechildren::-moz-tree-drop-feedback {
  background-color: #A1A1A1;
  width: 50px;
  height: 2px;
  margin-left: 5px;
}

/* ::::: tree progress meter ::::: */

treechildren::-moz-tree-progressmeter {
  margin: 2px 4px;
  border: 2px solid;
  -moz-border-top-colors: #AAAAAA #000000;
  -moz-border-right-colors: #FFFFFF #000000;
  -moz-border-bottom-colors: #FFFFFF #000000;
  -moz-border-left-colors: #AAAAAA #000000;
}

treechildren::-moz-tree-progressmeter(progressUndetermined) {
}

treechildren::-moz-tree-cell-text(progressmeter) {
  margin: 2px 4px;
  -moz-appearance: progressbar;
 }

/* ::::: tree columns ::::: */

treecol,
treecolpicker { 
  -moz-appearance: treeheadercell;
  -moz-box-align: center;
  -moz-box-pack: center;
  border: 2px solid;
  -moz-border-top-colors: ThreeDHighlight ThreeDLightShadow;
  -moz-border-right-colors: ThreeDDarkShadow ThreeDShadow;
  -moz-border-bottom-colors: ThreeDDarkShadow ThreeDShadow;
  -moz-border-left-colors: ThreeDHighlight ThreeDLightShadow;
  background-color: -moz-Dialog;
  color: -moz-DialogText;
  padding: 0px 4px;
}

.treecol-image {
  padding: 0px 1px;
}

.treecol-text {
  margin: 0px !important;
}

treecol[hideheader="true"] {
  -moz-appearance: none;
  border: none;
  padding: 0;
  max-height: 0px;
}

/* ..... internal box ..... */

treecol:hover:active,
treecolpicker:hover:active {
  border-top: 2px solid;
  border-right: 1px solid;
  border-bottom: 1px solid;
  border-left: 2px solid;
  -moz-border-top-colors: ThreeDDarkShadow ThreeDShadow;
  -moz-border-right-colors: ThreeDDarkShadow;
  -moz-border-bottom-colors: ThreeDDarkShadow;
  -moz-border-left-colors: ThreeDDarkShadow ThreeDShadow;
  background-color: #666666;
}

/* ::::: column drag and drop styles ::::: */

treecol[dragging="true"] {
  -moz-border-top-colors: ThreeDDarkShadow ThreeDShadow !important;
  -moz-border-right-colors: ThreeDDarkShadow ThreeDShadow!important;
  -moz-border-bottom-colors: ThreeDDarkShadow ThreeDShadow !important;
  -moz-border-left-colors: ThreeDDarkShadow ThreeDShadow !important;
  padding: 0px 4px !important;
  background-color: ThreeDShadow !important;
  color: ThreeDHighlight !important;
}

treecol[insertafter="true"] {
  -moz-border-right-colors: ThreeDDarkShadow ThreeDShadow;
}

treecol[insertbefore="true"] {
  -moz-border-left-colors: ThreeDDarkShadow ThreeDShadow;
}

treechildren::-moz-tree-column(insertbefore) {
  border-left: 1px solid ThreeDShadow;
}

treechildren::-moz-tree-column(insertafter) {
  border-right: 1px solid ThreeDShadow;
}

/* ::::: sort direction indicator :::::  */

.treecol-sortdirection {
  -moz-appearance: treeheadersortarrow;
  list-style-image: none;
}

.treecol-sortdirection[sortDirection="ascending"] {
  list-style-image: url("chrome://global/skin/tree/sort-asc.gif");
}

.treecol-sortdirection[sortDirection="descending"] {
  list-style-image: url("chrome://global/skin/tree/sort-dsc.gif");
}

/* ::::: column picker :::::  */

.tree-columnpicker-icon {
  list-style-image: url("chrome://global/skin/tree/columnpicker.gif");
}

/* ::::: twisty :::::  */

treechildren::-moz-tree-twisty {
  -moz-appearance: treetwisty;
  padding-right: 2px;
  width: 10px; /* The image's width is 10 pixels */
  list-style-image: url("chrome://global/skin/tree/twisty-clsd.gif");
}

treechildren::-moz-tree-twisty(open) {
  -moz-appearance: treetwistyopen;
  width: 10px; /* The image's width is 10 pixels */
  list-style-image: url("chrome://global/skin/tree/twisty-open.gif");
}

treechildren::-moz-tree-twisty(Name, separator) {
  -moz-appearance: none;
}

treechildren::-moz-tree-indentation {
  width: 16px;
}

/* ::::: gridline style ::::: */

treechildren.gridlines::-moz-tree-cell {
  border-right: 1px solid GrayText;
  border-bottom: 1px solid GrayText;
}

treechildren.gridlines::-moz-tree-row {
  border: none;
}

treechildren::-moz-tree-image(Name) {
  margin: 0px 4px;
  list-style-image: url("chrome://global/skin/tree/item.png");
}

treechildren::-moz-tree-image(Name, container) {
 list-style-image: url("chrome://global/skin/tree/folder.png");

}

treechildren::-moz-tree-image(Name, separator) {
  list-style-image: none;
  width: 0px !important;
  height: 0px !important;
  margin: 0px 0px;
}

treechildren::-moz-tree-cell-text(Name, separator) {
  color: #999;
  margin: 0 5px;
}

treechildren::-moz-tree-cell-text(Name, separator, selected, focus) {
  color: #FFF;
}

/* ::::: alternating background ::::: */

treechildren[alternatingbackground="true"]::-moz-tree-row(odd) {
  background-color: #f3f3f3;
}

treechildren[alternatingbackground="true"]::-moz-tree-row(odd, selected) {
  background-color: -moz-Dialog;
}

treechildren[alternatingbackground="true"]::-moz-tree-cell-text(odd, selected) {
  color: -moz-DialogText;
}

treechildren[alternatingbackground="true"]::-moz-tree-row(odd, selected, focus) {
  background-color: Highlight;
}

treechildren[alternatingbackground="true"]::-moz-tree-cell-text(odd, selected, focus) {
  color: HighlightText;
}
