/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Netscape Portable Runtime (NSPR).
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
** C++ access to NSPR locks (PRLock)
*/

#if defined(_RCLOCK_H)
#else
#define _RCLOCK_H

#include "rcbase.h"

#include <prlock.h>

class PR_IMPLEMENT(RCLock): public RCBase
{
public:
    RCLock();
    virtual ~RCLock();

    void Acquire();                 /* non-reentrant */
    void Release();                 /* should be by owning thread */

    friend class RCCondition;

private:
    RCLock(const RCLock&);          /* can't do that */
    void operator=(const RCLock&);  /* nor that */

    PRLock *lock;
};  /* RCLock */

/*
** Class: RCEnter
**
** In scope locks. You can only allocate them on the stack. The language
** will insure that they get released (by calling the destructor) when
** the thread leaves scope, even if via an exception.
*/
class PR_IMPLEMENT(RCEnter)
{
public:
    ~RCEnter();                     /* releases the lock */
    RCEnter(RCLock*);               /* acquires the lock */

private:
    RCLock *lock;

    RCEnter();
    RCEnter(const RCEnter&);
    void operator=(const RCEnter&);

    void *operator new(PRSize) { return NULL; }
    void operator delete(void*) { }
};  /* RCEnter */


inline RCEnter::RCEnter(RCLock* ml) { lock = ml; lock->Acquire(); }
inline RCEnter::~RCEnter() { lock->Release(); lock = NULL; }

#endif /* defined(_RCLOCK_H) */

/* RCLock.h */
