/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Kipp E.B. Hickman <kipp@netscape.com> (original author)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include <malloc.h>
#include "dict.h"

#ifdef __QNXNTO__
/* Need definition of NULL */
#include <stdio.h>
#endif

MallocDict::MallocDict(int nb)
{
    numBuckets = nb;
    buckets = (MallocDictEntry**) calloc(numBuckets, sizeof(MallocDictEntry*));
    rewind();
}

void MallocDict::rewind(void)
{
    iterNextBucket = -1;
    iterNextEntry = 0;
}

malloc_log_entry* MallocDict::next(void)
{
    if (iterNextEntry) {
	iterNextEntry = iterNextEntry->next;
    }
    while (!iterNextEntry) {
	iterNextBucket++;
	if (iterNextBucket >= numBuckets) {
	    return 0;
	}
	iterNextEntry = buckets[iterNextBucket];
    }
    return iterNextEntry->logEntry;
}

malloc_log_entry** MallocDict::find(u_long addr)
{
    u_long hash = addr % numBuckets;
    MallocDictEntry* mde = buckets[hash];
    while (mde) {
	if (mde->addr == addr) {
	    return &mde->logEntry;
	}
	mde = mde->next;
    }
    return 0;
}

void MallocDict::add(u_long addr, malloc_log_entry *lep)
{
    u_long hash = addr % numBuckets;
    MallocDictEntry** mdep = &buckets[hash];
    MallocDictEntry* mde = new MallocDictEntry;
    mde->addr = addr;
    mde->logEntry = lep;
    mde->next = *mdep;
    *mdep = mde;
}

void MallocDict::remove(u_long addr)
{
    u_long hash = addr % numBuckets;
    MallocDictEntry** mdep = &buckets[hash];
    MallocDictEntry* mde;

    while (NULL != (mde = *mdep)) {
	if (mde->addr == addr) {
	    *mdep = mde->next;
/*XXX	    delete mde; */
	    return;
	}
	mdep = &mde->next;
    }
}
